package com.ningdatech.pmapi.datascope.aop;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ningdatech.pmapi.datascope.model.DataScopeDTO;
import com.ningdatech.pmapi.datascope.model.DataScopeEntity;
import com.ningdatech.pmapi.datascope.provider.DataScopeContext;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.util.Objects;
import java.util.Optional;

/**
 * @program:
 * @description: 数据权限切面
 * LambdaDataScopeAspect
 * @author: PoffyZhang
 * @created: 2023/3/11 09:12
 */
@Aspect
@Component
@Slf4j
@AllArgsConstructor
@Order(10)
public class LambdaDataScopeAspect {

    @Pointcut("@annotation(com.ningdatech.pmapi.datascope.aop.LambdaDataScope)")
    public void lambdaDataScopeAspect() {

    }

    @Before("lambdaDataScopeAspect() && @annotation(lambdaDataScope)")
    public void before(JoinPoint joinPoint, LambdaDataScope lambdaDataScope) throws Throwable {
        Optional<DataScopeDTO> currentUserDataScoper = getCurrentUserDataScoper();
        if (!currentUserDataScoper.isPresent()) {
            return;
        }
        DataScopeDTO dataScopeDto = currentUserDataScoper.get();
        LambdaQueryWrapper<DataScopeEntity> wrapper =
                (LambdaQueryWrapper) joinPoint.getArgs()[lambdaDataScope.wrapperIndex()];

        if (Objects.isNull(wrapper)) {
            return;
        }

        switch (lambdaDataScope.type()) {
            case REGION:
                wrapper.in(CollUtil.isNotEmpty(dataScopeDto.getRegionCodes()), DataScopeEntity::getRegionCode, dataScopeDto.getRegionCodes());
                break;
            case ORG:
                wrapper.in(StringUtils.isNotBlank(dataScopeDto.getOrgCode()), DataScopeEntity::getOrgCode, dataScopeDto.getOrgCode());
                break;
            case EMPLOYEE:
                wrapper.in(StringUtils.isNotBlank(dataScopeDto.getEmployeeCode()), DataScopeEntity::getEmployeeCode, dataScopeDto.getEmployeeCode());
                break;
            case NONE:
                break;
            default:
                break;
        }

    }

    public Optional<DataScopeDTO> getCurrentUserDataScoper() {
        UserInfoDetails loginUser = LoginUserUtil.loginUserDetail();
        if (Objects.isNull(loginUser) || Objects.isNull(loginUser.getRoleCode())) {
            return Optional.empty();
        }
        return DataScopeContext.getDataScope(loginUser.getRoleCode().name());
    }
}